
(** On choisit ici de conserver le coin inférieur gauche et la dimension
    du cadre, mais ce n'est pas une obligation *)

type t = {
  llx: int; lly: int; (** lower-left corner *)
  width: int;
  height: int;
  kind: kind;
}

and kind =
  | Rectangle
  | Circle of int
  | Union of t * t

let width p = p.width
let height p = p.height

let rectangle x1 y1 x2 y2 =
  { llx = x1;
    lly = y1;
    width = x2 - x1;
    height = y2 - y1;
    kind = Rectangle }

let circle x y r =
  { llx = x - r;
    lly = y - r;
    width = 2 * r; height = 2 * r;
    kind = Circle r }

let union p1 p2 =
  let x1 = min p1.llx p2.llx in
  let y1 = min p1.lly p2.lly in
  let x2 = max (p1.llx + p1.width) (p2.llx + p2.width) in
  let y2 = max (p1.lly + p1.height) (p2.lly + p2.height) in
  { llx = x1; lly = y1; width = x2 - x1; height = y2 - y1;
    kind = Union (p1, p2) }

let rec move dx dy p =
  let kind =
    match p.kind with
    | Rectangle
    | Circle _ ->
        p.kind
    | Union (p1, p2) ->
        Union (move dx dy p1, move dx dy p2)
  in
  { llx = p.llx + dx;
    lly = p.lly + dy;
    width = p.width;
    height = p.height;
    kind = kind
  }

let rec draw p = match p.kind with
  | Rectangle -> Graphics.draw_rect p.llx p.lly p.width p.height
  | Circle r -> Graphics.draw_circle (p.llx + r) (p.lly + r) r
  | Union (p1, p2) -> draw p1; draw p2




